/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.cluster;

import es.prodevelop.android.spatialindex.quadtree.bucket.mr.MRBucketPRQuadtree;
import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.Point;

/**
 * A Cluster is group of points within a bounding box (Extent) and represented
 * by a Point, that can be for example the nearest point to the center of the
 * bounding box.
 * 
 * @author aromeu
 * 
 */
public class Cluster extends Point {

//	public static int EXPANDABLE_NUM_ITEMS = 100;

	private Extent boundingBox;
	private int numItems;
	private short level;
	private boolean isLeaf;
	private int ID;
	private short cat;
	private boolean expanded = false;

	/**
	 * Void constructor, needed by Perst
	 */
	public Cluster() {
	}

	/**
	 * 
	 * @param ID
	 *            The ID of the point that represents, that ID should be the
	 *            primary key on the row of the table where the Cluster is
	 *            stored
	 * @param boundingBox
	 *            The bounding box the Cluster groups the points
	 * @param medoid
	 *            The representant of the Cluster
	 * @param numItems
	 *            The number of items that are grouped
	 * @param level
	 *            The height level of the Cluster in a supposed ClusterTree
	 * @param isLeaf
	 *            If true, that Cluster is a leaf on a ClusterTree, so its items
	 *            can be retrieved from a spatial index given its bounding box
	 * @param cat
	 *            A short representing the category @see POICategories
	 */
	public Cluster(int ID, Extent boundingBox, Point medoid, int numItems,
			short level, boolean isLeaf, short cat) {
		super();
		this.ID = ID;
		this.boundingBox = boundingBox;
		this.setX(medoid.getX());
		this.setY(medoid.getY());
		this.numItems = numItems;
		this.level = level;
		this.isLeaf = isLeaf;
		this.cat = cat;
	}

	/**
	 * The MBR that contains of the points of the cluster
	 * 
	 * @return the boundingBox
	 */
	public Extent getBoundingBox() {
		return boundingBox;
	}

	/**
	 * @param boundingBox
	 *            the boundingBox to set
	 */
	public void setBoundingBox(Extent boundingBox) {
		this.boundingBox = boundingBox;
	}

	/**
	 * The number of points that the Cluster groups
	 * 
	 * @return the numItems
	 */
	public int getNumItems() {
		return numItems;
	}

	/**
	 * @param numItems
	 *            the numItems to set
	 */
	public void setNumItems(int numItems) {
		this.numItems = numItems;
	}

	/**
	 * @return the level
	 */
	public short getLevel() {
		return level;
	}

	/**
	 * @param level
	 *            the level to set
	 */
	public void setLevel(short level) {
		this.level = level;
	}

	/**
	 * @return the isLeaf
	 */
	public boolean isLeaf() {
		return isLeaf;
	}

	/**
	 * @param isLeaf
	 *            the isLeaf to set
	 */
	public void setLeaf(boolean isLeaf) {
		this.isLeaf = isLeaf;
	}

	/**
	 * The ID of the Cluster. That should be a primary key.
	 * 
	 * @return the iD
	 */
	public int getID() {
		return ID;
	}

	/**
	 * @param iD
	 *            the iD to set
	 */
	public void setID(int iD) {
		ID = iD;
	}

	public Object clone() {
		Cluster c = new Cluster();
		c.setBoundingBox((Extent) this.getBoundingBox().clone());
		c.setID(this.ID);
		c.setLeaf(isLeaf);
		c.setLevel(level);
		c.setNumItems(numItems);
		c.setVisible(this.isVisible());
		c.setX(getX());
		c.setY(getY());
		c.setCat(cat);

		return c;
	}

	public String toString() {
		return new StringBuffer().append(ID).append(";").append(getX())
				.append(";").append(getY()).append(";").append(numItems)
				.append(";").append(String.valueOf(!isLeaf)).append(";")
				.append(cat).append("|").toString();
	}

	public void destroy() {
		if (this.boundingBox != null)
			this.boundingBox.destroy();
		this.boundingBox = null;
	}

	public void setCat(short cat) {
		this.cat = cat;
	}

	public short getCat() {
		return cat;
	}

	public boolean isExpandable(Extent extent) {
		return getNumItems() < MRBucketPRQuadtree.EXPAND_CLUSTER_ITEMS
				|| (getNumItems() <= MRBucketPRQuadtree.DEFAULT_CLUSTER_ITEMS && (extent
						.getWidth() * 2 / 3 < getBoundingBox().getWidth() || extent
						.getHeight() * 2 / 3 < getBoundingBox().getHeight()));

	}

	public boolean isExpanded() {
		return expanded;
	}

	public void setExpanded(boolean expanded) {
		this.expanded = expanded;
	}
}
