/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.poi;

import java.util.ArrayList;

import org.garret.perst.Persistent;

public class Metadata extends Persistent {

	ArrayList categories = new ArrayList();

	public Metadata() {

	}

	public ArrayList getCategories() {
		return this.categories;
	}

	public void setCategories(ArrayList categories) {
		this.categories = categories;
	}

	public int getCategoryOffset(String category) {
		if (category.compareTo(POICategories.STREETS) == 0)
			return 0;

		int offset = 0;
		Metadata.Category cat = this.getCategory(category);
		if (cat != null) {
			final int size = this.categories.size();
			for (int i = 0; i < size; i++) {
				cat = (Metadata.Category) this.categories.get(i);
				if (cat.name.compareTo(category) == 0)
					return offset;
				offset += cat.total;
			}
			return offset;
		} else {
			offset = 0;
			final int size = this.categories.size();
			for (int i = 0; i < size; i++) {
				cat = (Metadata.Category) this.categories.get(i);
				final int length = cat.subcategories.size();
				Metadata.Subcategory s;
				for (int j = 0; j < length; j++) {
					s = (Metadata.Subcategory) cat.subcategories.get(j);
					if (s.name.compareTo(category) == 0)
						return offset;
					offset += s.total;
				}
			}
			return offset;
		}
	}

	public void addCategory(Metadata.Category category) {
		final int size = categories.size();

		boolean exists = false;
		for (int i = 0; i < size; i++) {
			if (((Metadata.Category) categories.get(i)).name
					.compareTo(category.name) == 0) {
				exists = true;
				break;
			}
		}

		if (!exists)
			categories.add(category);
	}

	public Indexed getCatOrSubcat(String catOrSubcat) {
		Metadata.Category cat = getCategory(catOrSubcat);
		if (cat != null)
			return cat;
		Metadata.Subcategory sub = getSubcategory(catOrSubcat);
		return sub;
	}

	public Metadata.Category getCategory(String category) {
		final int size = categories.size();
		Metadata.Category cat;
		for (int i = 0; i < size; i++) {
			cat = (Metadata.Category) categories.get(i);
			if (cat.name.compareTo(category) == 0) {
				return cat;
			}
		}
		return null;
	} 
	
	public Metadata.Category getCategoryForSubcategory(String subcategory) {
		final int size = categories.size();
		
		Metadata.Category cat;
		Metadata.Subcategory sub;
		ArrayList list;
		for (int i = 0; i < size; i++) {
			cat = (Metadata.Category) categories.get(i);
			list = cat.subcategories;
			final int length = list.size();
			for (int j = 0; j < length; j++) {
				sub = (Metadata.Subcategory) cat.subcategories.get(j);
				if (sub.name.compareTo(subcategory) == 0)
					return cat;
			}
		} 
		return null;
	}

	public Metadata.Subcategory getSubcategory(String subcategory) {
		final int size = categories.size();
		Metadata.Category cat;
		Metadata.Subcategory sub;
		ArrayList list;
		for (int i = 0; i < size; i++) {
			cat = (Metadata.Category) categories.get(i);
			list = cat.subcategories;
			final int length = list.size();
			for (int j = 0; j < length; j++) {
				sub = (Metadata.Subcategory) cat.subcategories.get(j);
				if (sub.name.compareTo(subcategory) == 0)
					return sub;
			}
		}
		return null;
	}

	public static class InitialNumber {
		public char initial;
		public short number;

		public InitialNumber() {

		}
	}

	public static class Indexed {
		public ArrayList initialNumbers = new ArrayList();
		public int total;
		public String name;

		public void addInitial(char c) {
			final int size = initialNumbers.size();

			boolean exists = false;
			Metadata.InitialNumber ini;
			for (int i = 0; i < size; i++) {
				ini = (Metadata.InitialNumber) initialNumbers.get(i);
				if (ini.initial == c) {
					ini.number++;
					exists = true;
					break;
				}
			}

			if (!exists) {
				ini = new Metadata.InitialNumber();
				ini.initial = c;
				ini.number = 1;
				initialNumbers.add(ini);
			}

			total++;
		}

		public void put(char arg0, short arg1) {
			InitialNumber i = new InitialNumber();
			i.initial = arg0;
			i.number = arg1;
			total += arg1;
			initialNumbers.add(i);
		}

		public Indexed() {
		}
	}

	public static class Category extends Indexed {
		public ArrayList subcategories = new ArrayList();

		public Category() {

		}

		public void addSubcategory(Subcategory subcategory) {
			final int size = subcategories.size();

			boolean contains = false;
			for (int i = 0; i < size; i++) {
				if (((Subcategory) subcategories.get(i)).name
						.compareTo(subcategory.name) == 0) {
					contains = true;
					break;
				}
			}

			if (!contains)
				subcategories.add(subcategory);
		}

		public Subcategory getSubCategory(String name) {
			final int size = subcategories.size();

			Subcategory s;
			for (int i = 0; i < size; i++) {
				s = (Subcategory) subcategories.get(i);
				if (s.name.compareTo(name) == 0)
					return s;
			}
			return null;
		}

		public void setSubcategories(ArrayList subcategories) {
			this.subcategories = subcategories;
		}

	}

	public static class Subcategory extends Indexed {

		public Subcategory() {

		}
	}
}
