/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.poi;

import es.prodevelop.gvsig.mini.utiles.Utilities;

/**
 * Represents a POI on the OSM database.
 * 
 * @author aromeu
 * 
 */
public class OsmPOI extends POI {

	private final static int ID_POS = 0;
	private final static int X_POS = 1;
	private final static int Y_POS = 2;
	private final static int CAT_POS = 3;
	private final static int SCAT_POS = 4;
	private final static int NAME_POS = 5;
	private final static int INFO_POS = 6;
	private final static int PHONE_POS = 7;
	private final static int MAIL_POS = 8;
	private final static int URL_POS = 9;
	private final static int WEB_POS = 10;
	private final static int WIKI_POS = 11;
	private final static int IMG_POS = 12;
	private final static int ADDR_POS = 13;

	private final static int NUM_PARMS = 14;

	private String category = "";
	private String subcategory = "";
	private String info = "";
	private String phone = "";
	private String email = "";
	private String url = "";
	private String website = "";
	private String wikipedia = "";
	private String image = "";
	private String address = "";

	/**
	 * Void constructor for Perst
	 */
	public OsmPOI() {
		super();
	}

	public OsmPOI(String category, String subcategory) {
		this.category = category;
		this.subcategory = subcategory;
	}

	/**
	 * 
	 * @param x
	 *            The x coordinate
	 * @param y
	 *            The y coordinate
	 * @param name
	 *            The description of the POI
	 */
	public OsmPOI(double x, double y, String name) {
		super(x, y, name);
	}

	/**
	 * 
	 * @param id
	 *            A unique ID, usually the ID of the row in the OSM database
	 * @param x
	 *            The x coordinate
	 * @param y
	 *            The y coordinate
	 * @param description
	 * @param category
	 *            one of @see POICategories
	 * @param subcategory
	 * @param info
	 * @param phone
	 * @param email
	 * @param url
	 * @param website
	 * @param wikipedia
	 * @param image
	 * @param address
	 */
	public OsmPOI(int id, double x, double y, String description,
			String category, String subcategory, String info, String phone,
			String email, String url, String website, String wikipedia,
			String image, String address) {
		super(x, y, description);
		this.setId(id);
		if (category != null)
			this.category = category;
		if (subcategory != null)
			this.subcategory = subcategory;
		this.info = info;
		this.phone = phone;
		this.email = email;
		this.url = url;
		this.website = website;
		this.wikipedia = wikipedia;
		this.image = image;
		this.address = address;
	}

	/**
	 * one of @see POICategories
	 * 
	 * @return the category
	 */
	public String getCategory() {
		return category;
	}

	/**
	 * @param category
	 *            the category to set
	 */
	public void setCategory(String category) {
		this.category = category;
	}

	/**
	 * @return the subcategory
	 */
	public String getSubcategory() {
		return subcategory;
	}

	/**
	 * @param subcategory
	 *            the subcategory to set
	 */
	public void setSubcategory(String subcategory) {
		this.subcategory = subcategory;
	}

	/**
	 * @return the info
	 */
	public String getInfo() {
		return info;
	}

	/**
	 * @param info
	 *            the info to set
	 */
	public void setInfo(String info) {
		this.info = info;
	}

	/**
	 * @return the phone
	 */
	public String getPhone() {
		return phone;
	}

	/**
	 * @param phone
	 *            the phone to set
	 */
	public void setPhone(String phone) {
		this.phone = phone;
	}

	/**
	 * @return the email
	 */
	public String getEmail() {
		return email;
	}

	/**
	 * @param email
	 *            the email to set
	 */
	public void setEmail(String email) {
		this.email = email;
	}

	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * @param url
	 *            the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/**
	 * @return the website
	 */
	public String getWebsite() {
		return website;
	}

	/**
	 * @param website
	 *            the website to set
	 */
	public void setWebsite(String website) {
		this.website = website;
	}

	/**
	 * @return the wikipedia
	 */
	public String getWikipedia() {
		return wikipedia;
	}

	/**
	 * @param wikipedia
	 *            the wikipedia to set
	 */
	public void setWikipedia(String wikipedia) {
		this.wikipedia = wikipedia;
	}

	/**
	 * @return the image
	 */
	public String getImage() {
		return image;
	}

	/**
	 * @param image
	 *            the image to set
	 */
	public void setImage(String image) {
		this.image = image;
	}

	/**
	 * @return the address
	 */
	public String getAddress() {
		return address;
	}

	/**
	 * @param address
	 *            the address to set
	 */
	public void setAddress(String address) {
		this.address = address;
	}

	/**
	 * The order of the attributes is:
	 * id,x,y,category,subcategory,name,info,phone
	 * ,email,url,website,wikipedia,image,address Where the "," is substituted
	 * by @see {@link POI#POI_SEPARATOR}
	 */
	public String toString() {
		return new StringBuffer().append(this.getId())
				.append(POI.POI_SEPARATOR).append(this.getX())
				.append(POI.POI_SEPARATOR).append(this.getY())
				.append(POI.POI_SEPARATOR).append(this.getCategory())
				.append(POI.POI_SEPARATOR).append(this.getSubcategory())
				.append(POI.POI_SEPARATOR).append(this.getDescription())
				.append(POI.POI_SEPARATOR).append(info)
				.append(POI.POI_SEPARATOR).append(phone)
				.append(POI.POI_SEPARATOR).append(email)
				.append(POI.POI_SEPARATOR).append(url)
				.append(POI.POI_SEPARATOR).append(website)
				.append(POI.POI_SEPARATOR).append(wikipedia)
				.append(POI.POI_SEPARATOR).append(image)
				.append(POI.POI_SEPARATOR).append(address).toString();
	}

	/**
	 * Parses a line like the one that returns {@link #toString()}
	 * 
	 * @param poiLine
	 * @return
	 */
	public static OsmPOI parse(String poiLine) {
		try {
			String[] parms = poiLine.split(POI.POI_SEPARATOR);

			if (parms == null)
				return null;

			String category = null;
			String subcategory = null;
			String info = null;
			String phone = null;
			String email = null;
			String url = null;
			String website = null;
			String wikipedia = null;
			String image = null;
			String address = null;
			int id = -1;
			double x = 0;
			double y = 0;
			String name = null;

			try {
				id = Integer.valueOf(parms[ID_POS]).intValue();
			} catch (Exception ignore) {

			}

			try {
				x = Double.valueOf(parms[X_POS]).doubleValue();
			} catch (Exception ignore) {

			}

			try {
				y = Double.valueOf(parms[Y_POS]).doubleValue();
			} catch (Exception ignore) {

			}

			try {
				name = parms[NAME_POS];
			} catch (Exception ignore) {

			}

			try {
				category = parms[CAT_POS];
			} catch (Exception ignore) {

			}

			try {
				subcategory = parms[SCAT_POS];
			} catch (Exception ignore) {

			}

			try {
				info = parms[INFO_POS];
			} catch (Exception ignore) {

			}

			try {
				phone = parms[PHONE_POS];
			} catch (Exception ignore) {

			}

			try {
				email = parms[MAIL_POS];
			} catch (Exception ignore) {

			}

			try {
				url = parms[URL_POS];
			} catch (Exception ignore) {

			}

			try {
				website = parms[WEB_POS];
			} catch (Exception ignore) {

			}

			try {
				wikipedia = parms[WIKI_POS];
			} catch (Exception ignore) {

			}

			try {
				image = parms[IMG_POS];
			} catch (Exception ignore) {

			}

			try {
				address = parms[ADDR_POS];
			} catch (Exception ignore) {

			}

			if (name != null && name.length() > 0) {

			} else {
				name = "";
			}

			OsmPOI poi = new OsmPOI(id, x, y, name, category, subcategory,
					info, phone, email, url, website, wikipedia, image, address);

			return poi;

			// return null;

		} catch (Exception e) {
			// logger.error("parse OsmPOIStreet", e);
			return null;
		}
	}

	public void destroy() {
		category = null;
		subcategory = null;
		info = null;
		phone = null;
		email = null;
		url = null;
		website = null;
		wikipedia = null;
		image = null;
		address = null;
		super.destroy();
	}

	public String getDescription() {
		try {
			String desc = Utilities.capitalizeFirstLetters(super
					.getDescription() != null ? super.getDescription() : OsmPOI.DEFAULT_DESCRIPTION);
			return desc;
		} catch (Exception e) {
			return DEFAULT_DESCRIPTION;
		}
	}

	public Object clone() {
		return new OsmPOI(getId(), getX(), getY(), getDescription(), category,
				subcategory, info, phone, email, url, website, wikipedia,
				image, address);
	}
}
