/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.poi;

import es.prodevelop.gvsig.mini.geom.Point;

/**
 * The base class to represent a POI. A point with a description. Coordinates
 * are expressed in EPSG:4326
 * 
 * @author aromeu
 */
public class POI extends Point {

	private int id = -1;
	private String description = DEFAULT_DESCRIPTION;
	public final static String DEFAULT_DESCRIPTION = "??";

	public final static String POI_SEPARATOR = "_SEP_";

	/**
	 * Void constructor for Perst
	 */
	public POI() {

	}

	public POI(String description) {
		this.description = description;
	}

	public POI(double x, double y, String description) {
		super(x, y);
		this.setDescription(description);
	}

	public String toString() {
		return getX() + POI_SEPARATOR + getY() + POI_SEPARATOR
				+ this.getDescription();
	}

	public String getDescription() {		
		return description;
	}

	public void setDescription(String description) {
		if (description != null && description.length() > 0)
			this.description = description;
	}

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @param id
	 *            the id to set
	 */
	public void setId(int id) {
		this.id = id;
	}

	public void destroy() {
		this.description = null;
	}
	
	public Object clone() {
		return new POI(this.getX(), this.getY(), this.getDescription());
	}
}
