/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.bucket;

import java.util.Vector;

import es.prodevelop.android.spatialindex.quadtree.QuadtreeAddNodeListener;

public class BucketPRQuadtree {

	private BucketPRQuadtreeNode top;
	private int totalNodes = 0;

	public final static int DEFAULT_BUCKET_ITEMS = 100;

	private QuadtreeAddNodeListener listener;

	public BucketPRQuadtree(double xMax, double xMin, double yMax, double yMin,
			int maxItems, double minSize) {
		top = new BucketPRQuadtreeNode(xMax, xMin, yMax, yMin, maxItems,
				minSize);
	}

	public void setQuadtreeNodeListener(QuadtreeAddNodeListener listener) {
		this.listener = listener;
	}

	public boolean addNode(double x, double y, Object obj) {
		boolean res = getTop().addNode(x, y, obj);
		if (res) {
			incrementTotalNodes();
			if (listener != null)
				listener.onNodeAdded(obj);
		}

		return res;
	}

	public Object removeNode(double x, double y, Object obj) {
		Object o = getTop().removeNode(x, y, obj);
		if (o != null)
			totalNodes--;
		return o;
	}

	public Object getNode(double x, double y) {
		return getTop().getNode(x, y);
	}

	public Object getNode(double x, double y, double withinDistance) {
		return getTop().getNode(x, y, withinDistance);
	}

	public Vector getNode(double xMax, double xMin, double yMax, double yMin) {
		return getNode(xMax, xMin, yMax, yMin, new Vector());
	}

	public Vector getNode(double xMax, double xMin, double yMax, double yMin,
			Vector nodes) {
		if (nodes == null)
			nodes = new Vector(10);

		if (xMin > xMax || (Math.abs(xMin - xMax) < 1e-6))
			return getTop().getNode(xMax, xMin, yMax, yMin,
					getTop().getNode(xMax, 0, yMax, yMin, nodes));
		else
			return getTop().getNode(xMax, xMin, yMax, yMin, nodes);
	}

	public Vector getNodes(double x, double y, double radius) {
		return getTop().getNodes(x, y, radius);
	}

	public Vector getNodes(double x, double y, double MinRadius,
			double MaxRadius) {
		return getTop().getNodes(x, y, MinRadius, MaxRadius);
	}

	public void clear() {
		totalNodes = 0;
		getTop().clear();
	}

	public void setTotalNodes(int totalNodes) {
		this.totalNodes = totalNodes;
	}

	public int getTotalNodes() {
		return totalNodes;
	}

	void incrementTotalNodes() {
		this.totalNodes++;
	}

	public BucketPRQuadtreeNode getTop() {
		return this.top;
	}

}
