/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.bucket;

import java.util.Vector;

import es.prodevelop.android.spatialindex.quadtree.QuadtreeBox;
import es.prodevelop.android.spatialindex.quadtree.QuadtreeLeaf;

public class BucketPRQuadtreeNode {

	public final double NO_MIN_SIZE = -1;
	public final double DEFAULT_MIN_SIZE = 5;

	public Vector items;
	private BucketPRQuadtreeNode[] branch;
	protected int maxItems;
	protected double minSize;
	public QuadtreeBox bounds;
	public boolean allTheSamePoint;
	public double firstX;
	public double firstY;
	private double lasteHalf = Double.MIN_VALUE;
	private double lastfHalf = Double.MIN_VALUE;

	public BucketPRQuadtreeNode(double xMax, double xMin, double yMax,
			double yMin, int maximumItems, double minimumSize) {
		bounds = new QuadtreeBox(xMax, xMin, yMax, yMin);
		maxItems = maximumItems;
		minSize = minimumSize;
		items = new Vector();
	}

	public boolean hasChildren() {
		if (getBranch() != null)
			return true;
		else
			return false;
	}

	protected void split() {
		// Make sure we're bigger than the minimum, if we care,
		if (minSize != NO_MIN_SIZE)
			if (Math.abs(bounds.Right - bounds.Left) < minSize
					&& Math.abs(bounds.Front - bounds.Back) < minSize)
				return;

		double ewHalf = (double) (bounds.Right - (bounds.Right - bounds.Left) * 0.5);
		double fbHalf = (double) (bounds.Front - (bounds.Front - bounds.Back) * 0.5);
		
//		System.out.println(ewHalf + ", " + fbHalf);
//		System.out.println(lasteHalf + ", " + lastfHalf);
		if (lasteHalf == ewHalf || lastfHalf == fbHalf) {
			allTheSamePoint = true;
			return;
		} else {
			lasteHalf = ewHalf;
			lastfHalf = fbHalf;
		}

		setBranch(instantiateBranch(ewHalf, fbHalf));

		QuadtreeLeaf[] temp = new QuadtreeLeaf[items.size()];
		items.copyInto(temp);

		items.removeAllElements();
		int size = temp.length;
		for (int i = 0; i < size; i++) {
			addNode(temp[i]);
		}
	}

	protected BucketPRQuadtreeNode getChild(QuadtreeLeaf leaf) {
		if (bounds.pointWithinBounds(leaf.getFx(), leaf.getFy())) {
			if (getBranch() != null) {
				for (int i = 0; i < getBranch().length; i++)
					if (getBranch()[i].bounds.pointWithinBounds(leaf.getFx(),
							leaf.getFy())) {
						updateValues(leaf.getFx(), leaf.getFy(),
								leaf.getObjectValue());
						return getBranch()[i].getChild(leaf);
					} else {
						int p = 0;
					}
			} else
				return this;
		}
		return null;
	}

	protected void updateValues(double x, double y, Object point) {

	}

	public boolean addNode(double x, double y, Object obj) {
		return addNode(new QuadtreeLeaf(x, y, obj));
	}

	protected boolean addNode(QuadtreeLeaf leaf) {
		if (getBranch() == null) {
			this.items.addElement(leaf);

			if (this.items.size() > maxItems && !this.allTheSamePoint)
				split();
			return true;
		} else {
			BucketPRQuadtreeNode node = getChild(leaf);
			if (node != null) {
				return node.addNode(leaf);
			} else {
				int i = 0;
			}
		}
		return false;
	}

	public Object removeNode(double x, double y, Object obj) {
		return removeNode(new QuadtreeLeaf(x, y, obj));
	}

	protected Object removeNode(QuadtreeLeaf leaf) {
		if (getBranch() == null) {
			// This must be the node that has it...
			for (int i = 0; i < items.size(); i++) {
				QuadtreeLeaf qtl = (QuadtreeLeaf) items.elementAt(i);
				if (leaf.getObjectValue() == qtl.getObjectValue()) {
					items.removeElementAt(i);
					return qtl.getObjectValue();
				}
			}
		} else {
			BucketPRQuadtreeNode node = getChild(leaf);
			if (node != null) {
				return node.removeNode(leaf);
			}
		}
		return null;
	}

	public Object getNode(double x, double y) {
		return getNode(x, y, Double.POSITIVE_INFINITY);
	}

	public Object getNode(double x, double y, double ShortestDistance) {
		double distance;
		Object closest = null;
		if (getBranch() == null) {
			int size = this.items.size();
			QuadtreeLeaf leaf;
			for (int i = 0; i < size; i++) {
				leaf = (QuadtreeLeaf) this.items.elementAt(i);
				distance = Math.sqrt(((x - leaf.getFx()) * (x - leaf.getFx()))
						+ ((y - leaf.getFy()) * (y - leaf.getFy())));

				if (distance < ShortestDistance) {
					ShortestDistance = distance;
					closest = leaf.getObjectValue();
				}
			}
			return closest;
		} else {
			// Check the distance of the bounds of the branch,
			// versus the bestDistance. If there is a boundary that
			// is closer, then it is possible that another node has an
			// Object that is closer.
			for (int i = 0; i < getBranch().length; i++) {
				double childDistance = getBranch()[i].bounds.borderDistance(x,
						y);
				if (childDistance < ShortestDistance) {
					Object test = getBranch()[i]
							.getNode(x, y, ShortestDistance);
					if (test != null)
						closest = test;
				}
			}
		}
		return closest;
	}

	public Vector getNode(double xMax, double xMin, double yMax, double yMin) {
		return getNode(new QuadtreeBox(xMax, xMin, yMax, yMin), new Vector());
	}

	public Vector getNode(double xMax, double xMin, double yMax, double yMin,
			Vector nodes) {
		return getNode(new QuadtreeBox(xMax, xMin, yMax, yMin), nodes);
	}

	public Vector getNode(QuadtreeBox rect, Vector nodes) {
		if (getBranch() == null) {
			int size = this.items.size();
			for (int i = 0; i < size; i++) {
				QuadtreeLeaf qtl = (QuadtreeLeaf) items.elementAt(i);
				if (rect.pointWithinBounds(qtl.getFx(), qtl.getFy()))
					nodes.addElement(qtl.getObjectValue());
			}
		} else {
			for (int i = 0; i < getBranch().length; i++) {
				if (getBranch()[i].bounds.within(rect))
					getBranch()[i].getNode(rect, nodes);
			}
		}
		return nodes;
	}

	public Vector getNodes(double x, double y, double radius) {
		Vector Nodes = new Vector();
		double distance;
		if (getBranch() == null) {
			int size = this.items.size();
			QuadtreeLeaf leaf;
			for (int i = 0; i < size; i++) {
				leaf = (QuadtreeLeaf) this.items.elementAt(i);
				distance = Math.sqrt(((x - leaf.getFx()) * (x - leaf.getFx()))
						+ ((y - leaf.getFy()) * (y - leaf.getFy())));

				if (distance < radius)
					Nodes.addElement(leaf.getObjectValue());
			}
			return Nodes;
		} else {
			// Check the distance of the bounds of the branch,
			// versus the bestDistance. If there is a boundary that
			// is closer, then it is possible that another node has an
			// Object that is closer.
			for (int i = 0; i < getBranch().length; i++) {
				double childDistance = getBranch()[i].bounds.borderDistance(x,
						y);

				if (childDistance < radius) {
					Object test = getBranch()[i].getNode(x, y, radius);
					if (test != null)
						Nodes.addElement(test);
				}

			}
		}
		return Nodes;
	}

	public Vector getNodes(double x, double y, double MinRadius,
			double MaxRadius) {
		Vector Nodes = new Vector();
		double distance, minDistance = 1e32;
		if (getBranch() == null) {
			int size = this.items.size();
			QuadtreeLeaf leaf;
			for (int i = 0; i < size; i++) {
				leaf = (QuadtreeLeaf) this.items.elementAt(i);
				distance = Math.sqrt(((x - leaf.getFx()) * (x - leaf.getFx()))
						+ ((y - leaf.getFy()) * (y - leaf.getFy())));

				if (distance >= MinRadius && distance < MaxRadius) {
					// if (distance <= minDistance) //closest Object first
					// {
					// Nodes.Insert(0, leaf.getObjectValue());
					// minDistance = Math.Min(minDistance, distance);
					// }
					// else
					Nodes.addElement(leaf.getObjectValue());
				}

			}
			return Nodes;
		} else {
			// Check the distance of the bounds of the branch,
			// versus the bestDistance. If there is a boundary that
			// is closer, then it is possible that another node has an
			// Object that is closer.
			for (int i = 0; i < getBranch().length; i++) {
				double childDistance = getBranch()[i].bounds.borderDistance(x,
						y);

				if (childDistance > MinRadius && childDistance <= MaxRadius) {
					Object test = getBranch()[i].getNode(x, y, MinRadius);
					if (test != null)
						Nodes.addElement(test);
				}

			}
		}
		return Nodes;
	}

	public void clear() {
		this.items.removeAllElements();
		if (getBranch() != null) {
			for (int i = 0; i < getBranch().length; i++) {
				getBranch()[i].clear();
			}
			branch = null;
		}
	}

	public BucketPRQuadtreeNode[] getBranch() {
		return branch;
	}

	public void setBranch(BucketPRQuadtreeNode[] branch) {
		this.branch = branch;
	}

	public BucketPRQuadtreeNode[] instantiateBranch(double ewHalf, double fbHalf) {
		BucketPRQuadtreeNode[] nodes = new BucketPRQuadtreeNode[4];
		nodes[0] = new BucketPRQuadtreeNode(ewHalf, bounds.Left, bounds.Front,
				fbHalf, maxItems, minSize); // left-front-top
		nodes[1] = new BucketPRQuadtreeNode(bounds.Right, ewHalf, bounds.Front,
				fbHalf, maxItems, minSize);
		nodes[2] = new BucketPRQuadtreeNode(ewHalf, bounds.Left, fbHalf,
				bounds.Back, maxItems, minSize);
		nodes[3] = new BucketPRQuadtreeNode(bounds.Right, ewHalf, fbHalf,
				bounds.Back, maxItems, minSize);
		return nodes;
	}

}
