/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.bucket.mr;

import es.prodevelop.android.spatialindex.poi.POI;
import es.prodevelop.android.spatialindex.quadtree.QuadtreeBox;
import es.prodevelop.android.spatialindex.quadtree.bucket.BucketPRQuadtreeNode;

/**
 * Chooses as a medoid the closest point to the center of the quadtree (of the
 * bounding box of a Cluster)
 * 
 * @see MRBucketPRQuadtree
 * @see Cluster
 * 
 * @author aromeu
 * 
 */
public class ClosestToQuadrantMedoidCandidateSelector implements
		MedoidCandidateSelector {

	private double[] medoidXY;

	/**
	 * Compares the distance from the candidate x-y and the current medoid x-y
	 * with the center of the cluster, and return True if the candidate is
	 * nearer than the current medoid
	 */
	public boolean isCandidate(double candidateX, double candidateY,
			double currentMedoidX, double currentMedoidY,
			BucketPRQuadtreeNode cluster) {
		final QuadtreeBox bounds = cluster.bounds;

		double centerX = ((bounds.Left + bounds.Right) / 2);
		double centerY = ((bounds.Back + bounds.Front) / 2);

		double distance = distance(centerX, centerY, candidateX, candidateY);
		double medoidDistance = distance(centerX, centerY, currentMedoidX,
				currentMedoidY);

		if (currentMedoidX == 0 && currentMedoidY == 0) {
			medoidXY = new double[] { candidateX, candidateY };
			return true;
		} else {
			if (distance < medoidDistance) {
				medoidXY = new double[] { candidateX, candidateY };
				return true;
			}
		}
		return false;
	}

	double distance(double x, double y, double xx, double yy) {
		double a1 = (x - xx) * (x - xx);

		// if (DEBUG)
		// System.out.println("a1: " + a1);
		//
		double a2 = (y - yy) * (y - yy);

		// if (DEBUG)
		// System.out.println("a2: " + a2);
		//
		double a3 = a1 + a2;

		// if (DEBUG)
		// System.out.println("a3: " + a3);
		//
		if (a3 < 0)
			return -1;

		double res = (double) (Math.sqrt(a3));

		// if (DEBUG)
		// System.out.println("res: " + res);
		//
		return res;
	}

	/**
	 * Extracts a valid ID from the candidate. If the candidate is a POI it
	 * takes the ID, otherwise returns -1
	 */
	public int getCandidateID(Object candidate) {
		if (candidate instanceof POI)
			return ((POI) candidate).getId();
		else
			return -1;
	}

	public double[] getMedoidXY(double candidateX, double candidateY,
			double currentMedoidX, double currentMedoidY,
			BucketPRQuadtreeNode cluster) {
		return medoidXY;
	}

	public MedoidCandidateSelector getSelector() {
		return new ClosestToQuadrantMedoidCandidateSelector();
	}

}
