/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.bucket.mr;

import es.prodevelop.android.spatialindex.quadtree.bucket.BucketPRQuadtree;
import es.prodevelop.android.spatialindex.quadtree.bucket.BucketPRQuadtreeNode;

/**
 * A BucketPRQuadtree that stores in each node the total number of items that
 * node groups, so this quadtree is useful to allow multi resolution
 * 
 * @author aromeu
 * 
 */
public class MRBucketPRQuadtree extends BucketPRQuadtree {

	private MRBucketPRQuadtreeNode top;

	public final static int DEFAULT_CLUSTER_ITEMS = 10;
	public final static int EXPAND_CLUSTER_ITEMS = 3;

	/**
	 * 
	 * @param xMax
	 *            xMax of the maximum bounding box of the quadtree
	 * @param xMin
	 *            xMin of the maximum bounding box of the quadtree
	 * @param yMax
	 *            yMax of the maximum bounding box of the quadtree
	 * @param yMin
	 *            yMin of the maximum bounding box of the quadtree
	 * @param maxItems
	 *            The max number of items in each intermediate node.
	 * @param minSize
	 *            The minimum size (height or width) of the bounding box of a
	 *            leaf node
	 * @param selector
	 *            A MedoidCandidateSelector to take a representant of each node
	 */
	public MRBucketPRQuadtree(double xMax, double xMin, double yMax,
			double yMin, int maxItems, double minSize,
			MedoidCandidateSelector selector) {
		super(xMax, xMin, yMax, yMin, maxItems, minSize);
		top = new MRBucketPRQuadtreeNode(xMax, xMin, yMax, yMin, maxItems,
				minSize, selector);
	}

	public BucketPRQuadtreeNode getTop() {
		return this.top;
	}

}
