/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.bucket.mr;

import java.util.ArrayList;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.GeometryFactory;

import es.prodevelop.android.spatialindex.quadtree.QuadtreeLeaf;
import es.prodevelop.android.spatialindex.quadtree.bucket.BucketPRQuadtreeNode;

/**
 * A node of a multi resolution quadtree. A node is split into 4 quadrants when
 * the number of items is higher than the maximum allowed for a node (remember
 * that is a 'bucket' quadtree).
 * 
 * A MRBucketPRQuadtreeNode stores the total number of items that hangs of it
 * and a representant, that is selected by an implementation of @see
 * MedoidCandidateSelector, and when got it's x-y balues and ID are store @see
 * POI
 * 
 * @author aromeu
 * 
 */
public class MRBucketPRQuadtreeNode extends BucketPRQuadtreeNode {

	private double medoidX;
	private double medoidY;

	private int medoidID;

	private int clusterNumItems;

	private MRBucketPRQuadtreeNode[] branch;
	private MedoidCandidateSelector selector;

	protected ArrayList points = new ArrayList();

	/**
	 * 
	 * @param xMax
	 *            xMax of the bounding box of this node
	 * @param xMin
	 *            xMin of the bounding box of this node
	 * @param yMax
	 *            yMax of the bounding box of this node
	 * @param yMin
	 *            yMin of the bounding box of this node
	 * @param maximumItems
	 *            The max number of items per bucket
	 * @param minimumSize
	 *            The minimum size of a side of the bounding box
	 * @param selector
	 *            An implementation of MedoidCandidateSelector to select a
	 *            representant of the cluster
	 */
	public MRBucketPRQuadtreeNode(double xMax, double xMin, double yMax,
			double yMin, int maximumItems, double minimumSize,
			MedoidCandidateSelector selector) {
		super(xMax, xMin, yMax, yMin, maximumItems, minimumSize);
		this.selector = selector.getSelector();
	}

	/**
	 * FIXME: Bad implementation
	 */
	public Object removeNode(double x, double y, Object obj) {
		Object o = removeNode(new QuadtreeLeaf(x, y, obj));

		// FIXME: Update medoid

		return o;
	}

	/**
	 * Check if the node point with x-y values is a candidate to be the
	 * representant of the cluster
	 * 
	 * @see MedoidCandidateSelector#isCandidate(double, double, double, double,
	 *      BucketPRQuadtreeNode)
	 */
	protected void updateValues(double x, double y, Object point) {
		// if (x != 0 && y != 0) {
		// points.add(new GeometryFactory().createPoint(new Coordinate(x, y)));
		// }
		if (selector.isCandidate(x, y, medoidX, medoidY, this)) {
			 double[] medoidXY = selector.getMedoidXY(x, y, medoidX, medoidY,
			 this);
			 medoidX = medoidXY[0];
			 medoidY = medoidXY[1];
//			medoidX = x;
//			medoidY = y;
			this.setMedoidID(selector.getCandidateID(point));
		}

		clusterNumItems++;
	}
	
	public void addElement(QuadtreeLeaf leaf) {
		QuadtreeLeaf l;
		boolean exists = false;
		for (int i = 0; i<items.size(); i++) {
			l = (QuadtreeLeaf)items.get(i);
			if (l.getFx() == leaf.getFx() && l.getFy() == leaf.getFy()) {
				exists = true;
				break;
			}				
		}
		
		if (!exists)
			items.addElement(leaf);		
	}

	// public boolean addNode(double x, double y, Object obj) {
	// boolean res = super.addNode(x, y, obj);
	//
	// // if (res) {
	// // updateValues(x, y);
	// // }
	//
	// return res;
	// }

	/**
	 * Adds a point to this node. If the number of items is higher than the max
	 * number of items of the bucket then, the node is split into 4 quadtrees
	 */
	protected boolean addNode(QuadtreeLeaf leaf) {		
		if (getBranch() == null) {
//			System.out.println(leaf.getObjectValue().toString());
			addElement(leaf);

			if (this.items.size() > maxItems && !this.allTheSamePoint)
//				try {
					split();
//				} catch (StackOverflowError e) {
//					System.out.println(e.getMessage());
//				}				
			else
				updateValues(leaf.getFx(), leaf.getFy(), leaf.getObjectValue());
			return true;
		} else {
			MRBucketPRQuadtreeNode node = (MRBucketPRQuadtreeNode) getChild(leaf);
			if (node != null) {
				return node.addNode(leaf);
			} else {
				int i = 0;
			}
		}
		return false;
	}

	/**
	 * Removes all the nodes from that node
	 */
	public void clear() {
		super.clear();
		reset();
	}

	/**
	 * Sets the mediodX-Y and the clusterNumItems to 0
	 */
	protected void reset() {
		clusterNumItems = 0;
		medoidX = 0;
		medoidY = 0;
	}

	/**
	 * @return the medoidX
	 */
	public double getMedoidX() {
		return medoidX;
	}

	/**
	 * @param medoidX
	 *            the medoidX to set
	 */
	public void setMedoidX(double medoidX) {
		this.medoidX = medoidX;
	}

	/**
	 * @return the medoidY
	 */
	public double getMedoidY() {
		return medoidY;
	}

	/**
	 * @param medoidY
	 *            the medoidY to set
	 */
	public void setMedoidY(double medoidY) {
		this.medoidY = medoidY;
	}

	/**
	 * @return the clusterNumItems
	 */
	public int getClusterNumItems() {
		if (clusterNumItems >= maxItems)
			return clusterNumItems - this.maxItems;

		return 0;

	}

	// /**
	// * @param totalNumItems
	// * the totalNumItems to set
	// */
	// public void setTotalNumItems(int totalNumItems) {
	// this.clusterNumItems = totalNumItems;
	// }

	public BucketPRQuadtreeNode[] getBranch() {
		return branch;
	}

	public void setBranch(BucketPRQuadtreeNode[] branch) {
		this.branch = (MRBucketPRQuadtreeNode[]) branch;
	}

	/**
	 * Splits this node into 4 quads
	 */
	public BucketPRQuadtreeNode[] instantiateBranch(double ewHalf, double fbHalf) {
		MRBucketPRQuadtreeNode[] nodes = new MRBucketPRQuadtreeNode[4];
		nodes[0] = new MRBucketPRQuadtreeNode(ewHalf, bounds.Left,
				bounds.Front, fbHalf, maxItems, minSize, selector); // left-front-top
		nodes[1] = new MRBucketPRQuadtreeNode(bounds.Right, ewHalf,
				bounds.Front, fbHalf, maxItems, minSize, selector);
		nodes[2] = new MRBucketPRQuadtreeNode(ewHalf, bounds.Left, fbHalf,
				bounds.Back, maxItems, minSize, selector);
		nodes[3] = new MRBucketPRQuadtreeNode(bounds.Right, ewHalf, fbHalf,
				bounds.Back, maxItems, minSize, selector);
		return nodes;
	}

	public void setMedoidID(int medoidID) {
		this.medoidID = medoidID;
	}

	public int getMedoidID() {
		return medoidID;
	}

}
