/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.bucket.mr;

import es.prodevelop.android.spatialindex.quadtree.bucket.BucketPRQuadtreeNode;

/**
 * Implementations of this interface should apply an algorithm to a new node
 * added to a MRBucketPRQuadtree to find out if it's a candidate to be the
 * medoid of the cluster or not.
 * 
 * Some implementations could be: The nearest point to the center of each node's
 * extent, an arbitrary point within a radius, the more relevant point within a
 * quadrant (supposing the are categorized or so...), etc.
 * 
 * @author aromeu
 * 
 */
public interface MedoidCandidateSelector {

	/**
	 * Decides wheter a candidate point should be selected as the medoid (the
	 * representant) of the current cluster node
	 * 
	 * @param candidateX
	 *            The x value of the candidate point
	 * @param candidateY
	 *            The y value of the candidate point
	 * @param currentMedoidX
	 *            The x value of the current selected medoid of the cluster
	 * @param currentMedoidY
	 *            The y value of the current selected medoid of the cluster
	 * @param cluster
	 *            The cluster where the candidate and the current medoid are
	 *            within
	 * @return True if the candidate point should replaced the current medoid.
	 */
	public boolean isCandidate(double candidateX, double candidateY,
			double currentMedoidX, double currentMedoidY,
			BucketPRQuadtreeNode cluster);
	
	/**
	 * Calculates the medoid for a given cluster
	 * 
	 * @param candidateX
	 *            The x value of the candidate point
	 * @param candidateY
	 *            The y value of the candidate point
	 * @param currentMedoidX
	 *            The x value of the current selected medoid of the cluster
	 * @param currentMedoidY
	 *            The y value of the current selected medoid of the cluster
	 * @param cluster
	 *            The cluster where the candidate and the current medoid are
	 *            within
	 * @return The x-y of the medoid of the cluster.
	 */
	public double[] getMedoidXY(double candidateX, double candidateY,
			double currentMedoidX, double currentMedoidY,
			BucketPRQuadtreeNode cluster);

	/**
	 * If the selected candidate is some kind of Object with an ID, then returns
	 * it
	 * 
	 * @param candidate
	 * @return
	 */
	public int getCandidateID(Object candidate);
	
	/**
	 * A factory method
	 * @return
	 */
	public MedoidCandidateSelector getSelector();

}
