/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.memory.cluster;

import es.prodevelop.android.spatialindex.cluster.Cluster;
import es.prodevelop.gvsig.mini.geom.Extent;

public class ClusterNode extends Cluster {

	private boolean hasBranch = false;
	private ClusterNode UR = null;
	private ClusterNode UL = null;
	private ClusterNode BR = null;
	private ClusterNode BL = null;	

	public Extent[] getExtents(Extent bounds) {
		double ewHalf = (double) (bounds.getMaxX() - (bounds.getMaxX() - bounds
				.getMinX()) * 0.5);
		double fbHalf = (double) (bounds.getMaxY() - (bounds.getMaxY() - bounds
				.getMinY()) * 0.5);

		Extent[] branch = new Extent[4];

		branch[0] = new Extent(bounds.getMinX(), fbHalf, ewHalf,
				bounds.getMaxY());
		branch[1] = new Extent(ewHalf, fbHalf, bounds.getMaxX(),
				bounds.getMaxY());
		branch[2] = new Extent(bounds.getMinX(), bounds.getMinY(), ewHalf,
				fbHalf);
		branch[3] = new Extent(ewHalf, bounds.getMinY(), bounds.getMaxX(),
				fbHalf);

		return branch;
	}

	public Extent[] getExtents() {
		final Extent bounds = this.getBoundingBox();
		double ewHalf = (double) (bounds.getMaxX() - (bounds.getMaxX() - bounds
				.getMinX()) * 0.5);
		double fbHalf = (double) (bounds.getMaxY() - (bounds.getMaxY() - bounds
				.getMinY()) * 0.5);

		Extent[] branch = new Extent[4];

		branch[0] = new Extent(bounds.getMinX(), fbHalf, ewHalf,
				bounds.getMaxY());
		branch[1] = new Extent(ewHalf, fbHalf, bounds.getMaxX(),
				bounds.getMaxY());
		branch[2] = new Extent(bounds.getMinX(), bounds.getMinY(), ewHalf,
				fbHalf);
		branch[3] = new Extent(ewHalf, bounds.getMinY(), bounds.getMaxX(),
				fbHalf);

		return branch;
	}

	public Extent[] getNodeExtents() {
		Extent[] branch = new Extent[4];
		if (hasBranch) {
			branch[0] = UL.getBoundingBox();
			branch[1] = UR.getBoundingBox();
			branch[2] = BL.getBoundingBox();
			branch[3] = BR.getBoundingBox();
		} else {
			branch = getExtents();
		}

		return branch;
	}

	/**
	 * @return the hasBranch
	 */
	public boolean isHasBranch() {
		return hasBranch;
	}

	/**
	 * @param hasBranch
	 *            the hasBranch to set
	 */
	public void setHasBranch(boolean hasBranch) {
		this.hasBranch = hasBranch;
		this.setLeaf(!hasBranch);
	}

	/**
	 * @return the uR
	 */
	public ClusterNode getUR() {
		return UR;
	}

	/**
	 * @param uR
	 *            the uR to set
	 */
	public void setUR(ClusterNode uR) {
		UR = uR;
	}

	/**
	 * @return the uL
	 */
	public ClusterNode getUL() {
		return UL;
	}

	/**
	 * @param uL
	 *            the uL to set
	 */
	public void setUL(ClusterNode uL) {
		UL = uL;
	}

	/**
	 * @return the bR
	 */
	public ClusterNode getBR() {
		return BR;
	}

	/**
	 * @param bR
	 *            the bR to set
	 */
	public void setBR(ClusterNode bR) {
		BR = bR;
	}

	/**
	 * @return the bL
	 */
	public ClusterNode getBL() {
		return BL;
	}

	/**
	 * @param bL
	 *            the bL to set
	 */
	public void setBL(ClusterNode bL) {
		BL = bL;
	}

	public Object clone() {
		ClusterNode node = new ClusterNode();
		node.setBL(this.getBL());
		node.setBoundingBox(this.getBoundingBox());
		node.setBR(this.getBR());
		node.setHasBranch(this.hasBranch);
		node.setID(this.getID());
		node.setLeaf(this.isLeaf());
		node.setLevel(this.getLevel());
		node.setNumItems(this.getNumItems());
		node.setUL(this.getUL());
		node.setUR(this.getUR());
		node.setX(this.getX());
		node.setY(this.getY());
		node.setCat(this.getCat());
		return node;
	}

	public void destroy() {
		super.destroy();

		if (this.UR != null)
			UR.destroy();

		if (this.BR != null)
			BR.destroy();

		if (this.BL != null)
			BL.destroy();

		if (this.UL != null)
			UL.destroy();

		UR = null;
		BR = null;
		BL = null;
		UL = null;
	}	
}
