/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.memory.cluster;

import java.util.ArrayList;
import java.util.Collection;

import es.prodevelop.android.spatialindex.quadtree.bucket.mr.MRBucketPRQuadtree;
import es.prodevelop.gvsig.mini.geom.Extent;

public class ClusterTree {

	private Extent extent;
	private ClusterNode root;

	// private Collection nodes;

	private int maxClusterLevel;
	private int maxClusterSize;

	public final static int UNKNOWN = -1;

	private ClusterNode nodeByID = null;

	public ClusterTree(ClusterNode root, Extent bounds, int maxClusterLevel) {
		this.extent = bounds;
		this.root = root;
		this.maxClusterLevel = maxClusterLevel;
	}

	public ClusterNode getNodeByID(int id) {
		nodeByID = null;
		getNodeByID(root, id);
		return nodeByID;
	}

	public void getNodeByID(ClusterNode rootNode, int id) {
		if (rootNode.getID() == id)
			nodeByID = rootNode;

		if (rootNode.getBL() != null)
			getNodeByID(rootNode.getBL(), id);

		if (rootNode.getUL() != null)
			getNodeByID(rootNode.getUL(), id);

		if (rootNode.getBR() != null)
			getNodeByID(rootNode.getBR(), id);

		if (rootNode.getUR() != null)
			getNodeByID(rootNode.getUR(), id);
	}

	public Collection getNodes(Extent bounds, double resolution) {
		// if (nodes != null)
		// nodes.clear();
		// else
		maxClusterSize = 0;
		final ArrayList nodes = new ArrayList();
		this.getNodes(root, extent, bounds, resolution, nodes);
		return nodes;
	}

	private void getNodes(final ClusterNode rootNode, final Extent rootBounds,
			final Extent bounds, final double resolution, final ArrayList nodes) {

		if (rootNode == null || rootBounds == null || bounds == null
		/*
		 * || rootNode.isExpanded() /* || levels < 0
		 */) {
			return;
		}

		if (rootNode.isExpanded()) {
			return;
		}

		// System.out.println(rootBounds.toString());

		if (!rootBounds.intersect(bounds)) {
			return;
		}

		// long t1 = System.currentTimeMillis();
		Extent[] extents = rootNode.getNodeExtents();
		// System.out.println("Time to get nodes: "
		// + (System.currentTimeMillis() - t1));

		/** Recursivelly get pois **/
		if (extents[0] != null)
			if (extents[0].intersect(bounds)) {
				if ((extents[0].getWidth() <= resolution || !rootNode
						.isHasBranch()) && rootNode.isValid()) {

					addNode(nodes, rootNode);
				} else
					// System.out.println(extents[0].toString());
					this.getNodes(rootNode.getUL(), extents[0], bounds,
							resolution, nodes);
			}

		if (extents[1] != null)
			if (extents[1].intersect(bounds)) {
				if ((extents[1].getWidth() <= resolution || !rootNode
						.isHasBranch()) && rootNode.isValid()) {
					addNode(nodes, rootNode);
				}
				// System.out.println(extents[1].toString());
				else
					this.getNodes(rootNode.getUR(), extents[1], bounds,
							resolution, nodes);

			}

		if (extents[2] != null)
			if (extents[2].intersect(bounds)) {
				if ((extents[2].getWidth() <= resolution || !rootNode
						.isHasBranch()) && rootNode.isValid()) {
					addNode(nodes, rootNode);
				}
				// System.out.println(extents[1].toString());
				else
					this.getNodes(rootNode.getBL(), extents[2], bounds,
							resolution, nodes);

			}

		if (extents[3] != null)
			if (extents[3].intersect(bounds)) {
				if ((extents[3].getWidth() <= resolution || !rootNode
						.isHasBranch()) && rootNode.isValid()) {
					addNode(nodes, rootNode);
				}
				// System.out.println(extents[1].toString());
				else
					this.getNodes(rootNode.getBR(), extents[3], bounds,
							resolution, nodes);

			}
	}

	/**
	 * @return the extent
	 */
	public Extent getExtent() {
		return extent;
	}

	/**
	 * @param extent
	 *            the extent to set
	 */
	public void setExtent(Extent extent) {
		this.extent = extent;
	}

	/**
	 * @return the root
	 */
	public ClusterNode getRoot() {
		return root;
	}

	/**
	 * @param root
	 *            the root to set
	 */
	public void setRoot(ClusterNode root) {
		this.root = root;
	}

	/**
	 * @return the maxClusterLevel
	 */
	public int getMaxClusterLevel() {
		return maxClusterLevel;
	}

	/**
	 * @param maxClusterLevel
	 *            the maxClusterLevel to set
	 */
	public void setMaxClusterLevel(int maxClusterLevel) {
		this.maxClusterLevel = maxClusterLevel;
	}

	private void addNode(final ArrayList nodes, final ClusterNode node) {
		// if (this.nodes.contains(node))
		// return;
		// long t1 = System.currentTimeMillis();
		final int numItems = node.getNumItems();
		// if (numItems < MRBucketPRQuadtree.DEFAULT_CLUSTER_ITEMS) {
		// return;
		// }
		if (numItems > maxClusterSize)
			maxClusterSize = node.getNumItems();
		nodes.add(node);
		// System.out.println("Time to add nodes: "
		// + (System.currentTimeMillis() - t1));

	}

	public int getMaxClusterSize() {
		return maxClusterSize;
	}

}
