/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.android.spatialindex.quadtree.persist.perst;

import org.garret.perst.FieldIndex;
import org.garret.perst.Index;
import org.garret.perst.Persistent;
import org.garret.perst.SpatialIndexR2;
import org.garret.perst.Storage;
import org.garret.perst.fulltext.FullTextIndex;

import es.prodevelop.android.spatialindex.poi.POI;

/**
 * Root class to use on the Perst database. Contains Spatial indexes @see
 * {@link Storage#createSpatialIndexR2()} to store pois (@see OsmPOI) and
 * streets (@see OsmPOIStreet)
 * 
 * @author aromeu
 * 
 */
public class SpatialIndexRoot extends Persistent {

	private SpatialIndexR2 poiIndex;
	private SpatialIndexR2 streetSpatialIndex;
	private FieldIndex streetIndex;
	private Index metadataIndex;
	private FullTextIndex fullTextIndex;

	public final static String METADATA_STREET_KEY = "METADATA_STREET";
	public final static String METADATA_POI_KEY = "METADATA_POI";

	public final static String DEFAULT_LANGUAGE = "en";
	public final static int DEFAULT_MAX_RESULTS = 15000;
	public final static int DEFAULT_MAX_TIME = 15000;

	public SpatialIndexRoot(Storage db) {
		super(db);
		poiIndex = db.createSpatialIndexR2();
		streetSpatialIndex = db.createSpatialIndexR2();
		streetIndex = db.createRandomAccessFieldIndex(POI.class, "description",
				false);
		fullTextIndex = db.createFullTextIndex();
		metadataIndex = db.createIndex(String.class, true);
	}

	public SpatialIndexRoot() {
	}

	/**
	 * @return the poiIndex
	 */
	public SpatialIndexR2 getPoiIndex() {
		return poiIndex;
	}

	/**
	 * @param poiIndex
	 *            the poiIndex to set
	 */
	public void setPoiIndex(SpatialIndexR2 poiIndex) {
		this.poiIndex = poiIndex;
	}

	/**
	 * @return the streetIndex
	 */
	public FieldIndex getStreetIndex() {
		return streetIndex;
	}

	/**
	 * @param streetIndex
	 *            the streetIndex to set
	 */
	public void setStreetIndex(FieldIndex streetIndex) {
		this.streetIndex = streetIndex;
	}

	/**
	 * @return the metadataIndex
	 */
	public Index getMetadataIndex() {
		return metadataIndex;
	}

	/**
	 * @param metadataIndex
	 *            the metadataIndex to set
	 */
	public void setMetadataIndex(Index metadataIndex) {
		this.metadataIndex = metadataIndex;
	}

	public FullTextIndex getFullTextIndex() {
		return this.fullTextIndex;
	}

	public SpatialIndexR2 getStreetSpatialIndex() {
		return streetSpatialIndex;
	}

}
